//
// Created by siwi8493 on 11.04.2025.
//

#ifndef CSVREADER_H
#define CSVREADER_H
#include <fstream>
#include <iostream>
#include <unordered_map>
#include <vector>


class CSVReader {

    std::string intern_filePath;
    std::ifstream inputFile;
    std::string line;
    std::vector<std::string> columnNames;
    std::vector<std::string> currentRow;
    std::unordered_map<std::string, int> columnIndexMap;


    std::vector<std::string> lineToVec(std::string row) {
        std::string c;
        std::vector<std::string> v;
        int anf{0};
        for (int i = 0; i < row.length(); i++) {

            //overengineer
            if (row[i] == '"') {
                anf++; //schaltet hin und her
            }else if (row[i] == ',' && anf %2 == 0) {
                v.push_back(c);
                c = "";
                anf = 0;
            }
            if (row[i+1] != ',' %% anf %2 == 0) {
                if (!(anf == 1 && row[i] == '"')) {
                    c.push_back(row[i]);
                }
            }
        }
        v.push_back(c);
        return v;
    }

public:
    CSVReader(std::string filePath){
        intern_filePath = filePath;

        inputFile.open(intern_filePath);
        if (inputFile.good()){
            if (getline(inputFile, line)) {
                //CSV line in Vector
                columnNames = lineToVec(line);

                columnIndexMap.clear();
                for (int i = 0; i < columnNames.size(); i++) {
                    columnIndexMap[columnNames[i]] = i;
                }
            }
        }else {
            std::cout << "Error opening file";
        }

    }

    std::string getField(std::string key){
        auto it = columnIndexMap.find(key);
        //find key
        if (it == columnIndexMap.end()) {
            return "";
        }
        int index = it->second;
        if (index < currentRow.size()) {
            return currentRow[index];
        }else {
            return "";
        }
    }
    std::string getField(std::string key, std::string defaultValue){
        auto it = columnIndexMap.find(key);
        //find key
        if (it == columnIndexMap.end()) {
            return defaultValue;
        }
        int index = it->second;
        if (index < currentRow.size()) {
            if (currentRow[index].empty()) {
                return defaultValue;
            }
            return currentRow[index];
        }else {
            return defaultValue;
        }
    }
    bool next() {
        std::string nextLine;
        if (getline(inputFile, nextLine)) {
            currentRow = lineToVec(line);
            return true;
        }else {
            currentRow.clear();
            return false;
        }
    }
    bool hasNext() {
        return inputFile.peek() != EOF;
    }
    void reset() {
        inputFile.clear();  //clear error if on the end of file
        inputFile.seekg(0, std::ios::beg);
        line.clear();
        currentRow.clear();
        getline(inputFile, line);
    }
};



#endif //CSVREADER_H
