# Übung 1

## Dateien einlesen (nicht relevant für die Abgabe)
Zum Einlesen von Dateien verwenden Sie die Klasse ifstream (https://en.cppreference.com/w/cpp/io/basic_ifstream), welche für das Zeichen-weise Einlesen aus Dateien gedacht ist. Um die Datei anzugeben aus der gelesen werden soll wird der Pfad zu dieser Datei im Konstruktor oder der `open()`-Methode eines `ifstream`-Objekts übergeben (https://en.cppreference.com/w/cpp/io/basic_ifstream/open). \
Anschließend können bspw. mit der `getline()`-Funktion Zeilenweise die Eingabedatei ausgelesen werden.

Der folgende Code-Auszug liest eine Datei ein und gibt diese Zeilenweise wieder aus.
```c++
#include <iostream>
#include <fstream>
#include <string>

int main(int argc, char** argv) {    
  std::string path{argv[1]};    
  std::ifstream inputFile;    
  inputFile.open(path, std::ifstream::in);    
  std::string line;       
  
  while (inputFile.good() == true) {        
    getline(inputFile, line);        
    std::cout << line << std::endl;    
  }    
  
  return 0;
}
```

1. Klonen Sie das Repository der Übung
1. Laden Sie die GTFS-Daten herunter (https://cloud.bht-berlin.de/index.php/s/aTJTRQnYX7cgXga), entpacken diese und machen Sie sich mit den darin enthaltenen Dateien vertraut.
2. Lesen Sie die Dateien zeilenweise aus und zählen Sie die Zeilen pro Datei. Geben Sie das Ergebnis auf der Kommandozeile aus in dem Format:

```
agency.txt: XXX lines
calendar_dates.txt: XXX lines
calendar.txt: XXX lines...
```

3. Erweitern Sie die Anwendung und geben Sie außerdem an wie viele Spalten in der Datei sind.
```
agency.txt: XXX lines with Y columns
calendar_dates.txt: XXX lines with Y columns
calendar.txt: XXX lines with Y columns...
```

## CSV-Verarbeitung
Die Verarbeitung von CSV-Dateien ist immer wieder eine Anforderung in Anwendungen. Dabei ist das Dateiformat einfach aufgebaut. In den Beispiel-Dateien ist zu erkennen, dass mehrere Datenfelder pro Zeile angegeben sind, welche durch ein Komma (`,`) getrennt sind. Jede Zeile endet mit einem LF (Line Feed = `\n`) oder einem CRLF (Carriage Return + Line Feed = `\r\n`).

Es muss berücksichtigt werden, dass das Komma auch in regulären Texten vorkommen kann (bspw. "Eisenhüttenstadt, Bahnhof"). In diesen Fällen darf das Komma dann nicht als Trennzeichen zwischen Spalten interpretiert werden. Um diese Fälle zu implementieren werden Anführungszeichen verwendet (") um Kommata in Texten von Kommata als Steuerzeichen zu trennen (siehe RFC 4180 - https://datatracker.ietf.org/doc/html/rfc4180).

```csv
"de:13004:9005","","Schwerin, Hauptbahnhof",,"53.634739000000","11.407455000000",0,,0,""
```

In diesem Beispiel sehen Sie die Verwendung von " um Spalten mit Text-Daten zu markieren. Sie sehen ebenfalls leere Spalten durch die Verwendung von "" oder dadurch, dass keine Zeichen zwischen zwei Kommata vorkommen.

CSV-Dateien können dabei beliebig viele Spalten enthalten. Um mit effizient mit solchen Daten in C++ umzugehen gibt es mit `std::vector` (aus dem Header `<vector>`) einen sog. Container, in dem beliebig viele Elemente gespeichert werden können (wie ein Array, dessen Länge nicht bei der Definition angegeben werden muss und welches wachsen kann). Diese Klasse ist als sog. "Template" implementiert. Das bedeutet, dass der Datentyp der Elemente, welche in dem Container abgelegt werden nicht festgelegt ist, sondern durch uns zur Deklaration in spitzen Klammern angegeben werden kann.

```c++
#include <vector>
int main() {  
  std::vector<std::string> fields; // Ein Vector für Zeichenketten  
  std::vector<int> numbers;        // Ein Vector mit ganzen Zahlen  
  fields.push_back("Schwerin");    // push_back fügt Element am Ende des Vectors an  
  numbers.push_back(100);           
  
  fields.push_back(100);           // Compile-Error! fields nur std::string-Elemente  
  
  std::cout << fields[0] << std::endl;  // Zugriff auf Elemente wie bei Arrays  
  std::cout << fields.size() << std::endl;  // Anzahl Elemente mit .size()
}
```

1. Erstellen Sie eine Klasse zum Auslesen von CSV-Dateien mit dem Namen `CSVReader`.
2. Die Klasse besitzt einen Konstruktor, welcher einen `std::string` als Parameter übergeben bekommt mit dem Pfad der CSV-Datei, welche eingelesen werden soll.
3. Ihre Klasse liest die Kopf-Zeile der Datei ein ermittelt so die Spalten-Namen.
4. Die Klasse soll immer eine Zeile der CSV-Datei Zwischenspeichern und deren Felder über den Spaltennamen zugreifbar machen.

```c++
CSVReader myReader{"routes.txt"}; // Öffnet CSV-Datei routes.txt + liest Header
myReader.getField("route_id");    // Gibt die Daten aus der Spalte zurück, welche den Text      
                                  // route_id im Header besitzt
myReader.next();                  // liest die nächste Zeile ein
```

5. Ihre Klasse soll die folgenden Methoden besitzen:
    1. `std::string getField(std::string key)` - Gibt den Wert aus der Spalte mit der Überschrift key der aktuellen Zeile zurück. Existiert die Spalte nicht oder ist diese leer wird ein leerer String zurückgegeben.
    1. `std::string getField(std::string key, std::string defaultValue)` - Funktioniert wie die Methode aus 5.1), jedoch wird `defaultValue` zurückgegeben wenn die es die angeforderte Spalte nicht gibt oder der Inhalt leer ist.
    1. `bool next()` - Liest die nächste Zeile in der Datei ein. Gibt true zurück wenn es eine nächste Zeile gab oder false wenn bereits alle Zeilen gelesen wurden.
    1. `bool hasNext(`) - Prüft ob es noch eine nächste Zeile in der Datei ein. Gibt true zurück wenn es eine nächste Zeile gibts sonst false.
    1. `void reset()` - Springt zurück zur ersten Zeile mit Daten in der Eingabedatei.


## Verarbeitung von GTFS-Daten
In Ihrem Repository befindet sich eine Datei `types.h`, welche für die Daten in den CSV-Dateien passende Datenstrukturen definiert. Diese entsprechen der Definition des GTFS-Standards.

Für die Speicherung von mehreren Elementen haben Sie bereits `std::vector` verwendet. Für manche Daten ist es sinnvoll, diese zu einem Schlüssel zu speichern, um schnell über den Schlüssel darauf zugreifen zu können. Hierzu können wir `std::unordered_map` aus dem Header `<map>` verwenden.

Ähnlich wie `vector` ist diese Klasse eine Template-Klasse, benötigt aber zwei Datentypen - einer, welcher den Datentyp des Schlüssels spezifiziert, sowie einer für den Datentyp der Einträge, welche zu dem Schlüssel gespeichert werden.
```c++
std::unordered_map<std::string, Agency> agencies; // Schlüssel ist vom Typ std::string
                                                  // Werte vom Typ Agency (types.h)
Agency a = { "BVG", "Berliner Verkehrsges.", "bvg.de", "Europe/Berlin", "de", "" };
agencies["BVG"] = a; // Zugriff wieder über []
```

1. Erstellen Sie eine Klasse zum Verarbeiten aller GTFS-Dateien mit dem Namen `Network`.
1. Die Klasse besitzt einen Konstruktor, welcher einen std::string als Parameter übergeben bekommt mit dem Pfad zum Verzeichnis, in welchem die CSV-Dateien abgelegt sind.
1. Die Dateien werden eingelesen und verarbeitet. Diese sind anschließend über Attribute mit der Sichtbarkeit public abrufbar:
```c++
public:    
  std::unordered_map<std::string, Agency> agencies;    // Inhalt von agency.txt  
  std::vector<CalendarDate> calendarDates;             // Inhalt von calendar_dates.txt
  std::unordered_map<std::string, Calendar> calendars; // Inhalt von calendar.txt  
  std::unordered_map<std::string, Level> levels;       // Inhalt von levels.txt  
  std::unordered_map<std::string, Pathway> pathways;   // Inhalt von pathways.txt  
  std::unordered_map<std::string, Route> routes;       // Inhalt von routes.txt  
  std::vector<Shape> shapes;                           // Inhalt von shapes.txt  
  std::vector<StopTime> stopTimes;                     // Inhalt von stop_times.txt  
  std::unordered_map<std::string, Stop> stops;         // Inhalt von stops.txt  
  std::vector<Transfer> transfers;                     // Inhalt von transfers.txt  
  std::vector<Trip> trips;                             // Inhalt von trips.txt
```